<?php

define("DIR_TEMPLATING_DMK",dirname(__FILE__));
define("DIR_PLUGIN_TEMPLATING_DMK",DIR_TEMPLATING_DMK."/plugins/");

class DmkTemplate
  {
   
   public $template_dir = './templates/';
   public $compile_dir = './templates_c/';
   public $config_dir = './configs/';
   public $cache_dir = './templates_cache/';
   public $tpl_every_time_compile = false;//параметр для тестирования шаблонизатора. 
   //true - будет каждый раз компилировать , 
   //false - компилятор будет проверять время последнего редактирования файла шаблона (для обычной работы)
   
   public $tpl_vars = array();
   public $error_reporting = E_ALL;
   
   public $caching = 0;// при значении равном 1 - кэширование будет выполнено 1 раз, и после кэширование будет отключено. для постоянного кэширования нужно ставить значение true 
   public $cache_lifetime = 60;//время кэширования
   
   public function __construct()
     {
      $this->init();
     }
   
   public function init()
     {
      
     }
   
   public function assign($v,$v2 = NULL)
     {
      
      if ( is_array($v) )
        $this->tpl_vars = array_merge($this->tpl_vars,$v);
       else
         $this->tpl_vars[$v] = $v2;
      
     }
   
   public function display($nfileTpl,$updateCache = false,$vars = false)
     {
      echo( $this->fetch($nfileTpl,$updateCache,$vars) );
     }
   
   public function fetch($nfileTpl,$updateCache = false,$vars = false)
     {
      $tpl_content = "";
      if ($vars === false) $vars=$this->tpl_vars;
      $this->tpl_vars=array();
      
      $data_cachefile = array( "", false);//информация о кэшированном файле - путь к файлу и состояние кэша - true или false
      $path_cachefile = $this->get_cache_path($this->constr_nfile_compile($nfileTpl));
      if ( $this->caching && !$updateCache && $this->is_cached($nfileTpl)  )
        {
         $tpl_content = implode("", file($path_cachefile));
         if ( $this->caching === 1 )
           $this->caching = 0;
         return($tpl_content);
        }
      
      if ( $path_compfile = $this->compile($nfileTpl) )
        {
         
         //$vars = &$this->tpl_vars;
         $previous_error_reporting = -1;
         if( $this->error_reporting !== error_reporting() )
           {
            $previous_error_reporting = error_reporting();
            error_reporting($this->error_reporting);
           }
         
         ob_start();
         $content = ob_get_contents();
         ob_clean();
         
         $this->include_compfile($path_compfile,$vars);
         
         $tpl_content = ob_get_contents();
         ob_end_clean();
         echo($content);
         
         if ( $previous_error_reporting !== -1 )
           {
            error_reporting($previous_error_reporting);
           }
         
         if ( $this->caching )
           {
            $df = fopen($path_cachefile,"w");
            if ( $df )
              {
               fwrite($df,$tpl_content);
               fclose($df);
              }
             else
               {
                $tpl_content .= "<br><i>Ошибка при создании кэша файла ".$nfileTpl."</i><br>";
               }
            if ( $this->caching === 1 )
              $this->caching = 0;
           }
         
        }
       else
         {
          $tpl_content = "<br><i>Ошибка при компилировании файла ".$nfileTpl.".</i><br>";
         }
      
      return ($tpl_content);
     }
   
   private function include_compfile($path_compfile,$vars)
     {
      include($path_compfile);
     }
   
   public function compile($nfileTpl)
     {
      
      $s_compiled = $this->is_compiled($nfileTpl);
      
      $path_compfile = $s_compiled[0];
      
      if ( !$s_compiled[1] )
        {
         $path_fileTpl = $this->get_template_path($nfileTpl);
         $mas_fileTpl = file($path_fileTpl);
         
         if ( $mas_fileTpl )
           {
            
            require_once("dmk_compiler.class.php");
            $obj_compiler = new dmk_compiler;
            $text = $obj_compiler->compile_mas($mas_fileTpl,$nfileTpl);
            
            
            //echo($path_compfile);
            
            $df = fopen($path_compfile,"w");
            if ( !$df )
              {
               return (false);
              }
            
            if (fwrite($df,$text) === false)
              {
               return (false);
              }
            fclose($df);
            
           }
          else
            {
             return (false);
            }
         
        }
      
      return ($path_compfile);
     }
   
   function is_compiled($nfileTpl)
     {
      $path_compfile = $this->get_compile_path($this->constr_nfile_compile($nfileTpl));
      $on_compiled = false;
      $path_file = $this->get_template_path($nfileTpl);
      //if ( is_file($path_compfile) && file_exists($path_file) && ( filemtime($path_file) <= filemtime($path_compfile)  ) )
      if ( is_file($path_compfile) && file_exists($path_file) && ( $f1 = stat( $path_compfile ) ) && ( $f2 = stat( $path_file ) ) && ( $f2["mtime"] <= $f1["mtime"] ) )
        $on_compiled = !$this->tpl_every_time_compile;
      
      return (array($path_compfile,$on_compiled));
     }
   
   function get_template_path($nfileTpl)
     {
      //получаем путь к файлу шаблона
      return ($this->template_dir.$nfileTpl);
     }
   
   function constr_nfile_compile($nfileTpl)
     {
      //формируем имя скомпилированного файла для соответствующего шаблона
      $name = $nfileTpl.'.'.substr(abs(crc32($this->get_template_path($nfileTpl).$nfileTpl)),0,6).'.php';
      return ($name);
     }
   
   function get_compile_path($nameFile)
     {
      //получаем путь к скомпилированному файлу
      return ( $this->compile_dir . basename($nameFile) );
     }
   
   function is_cached($nfileTpl)
     {
      $path_cachefile = $this->get_cache_path($this->constr_nfile_compile($nfileTpl));
      
      $on_cache = false;
      //$path_file = $this->get_template_path($nfileTpl);
      //if ( is_file($path_compfile) && file_exists($path_file) && ( filemtime($path_file) <= filemtime($path_compfile)  ) )
      //  $on_compiled = !$this->tpl_every_time_compile;
      
      if ( $this->caching && is_file($path_cachefile) && ( ( $this->cache_lifetime == -1 ) || ( filemtime($path_cachefile) > time() - $this->cache_lifetime )  ) )
        $on_cache = true;
      
      return ($on_cache);
     }
   
   function get_cache_path($nameFile)
     {
      //получаем путь к файлу кэша
      return ( $this->cache_dir . basename($nameFile) );
     }
   
  }




